@@ -4,32 +4,36 @@ import os
 import shutil
 import textwrap
 
-from ..util.compat import u, has_pep3147, get_current_bytecode_suffixes
-from ..script import Script, ScriptDirectory
-from .. import util
 from . import engines
 from . import provision
+from .. import util
+from ..script import Script
+from ..script import ScriptDirectory
+from ..util.compat import get_current_bytecode_suffixes
+from ..util.compat import has_pep3147
+from ..util.compat import u
 
 
 def _get_staging_directory():
     if provision.FOLLOWER_IDENT:
         return "scratch_%s" % provision.FOLLOWER_IDENT
     else:
-        return 'scratch'
+        return "scratch"
 
 
 def staging_env(create=True, template="generic", sourceless=False):
     from alembic import command, script
+
     cfg = _testing_config()
     if create:
-        path = os.path.join(_get_staging_directory(), 'scripts')
+        path = os.path.join(_get_staging_directory(), "scripts")
         if os.path.exists(path):
             shutil.rmtree(path)
         command.init(cfg, path, template=template)
         if sourceless:
             try:
                 # do an import so that a .pyc/.pyo is generated.
-                util.load_python_file(path, 'env.py')
+                util.load_python_file(path, "env.py")
             except AttributeError:
                 # we don't have the migration context set up yet
                 # so running the .env py throws this exception.
@@ -38,10 +42,13 @@ def staging_env(create=True, template="generic", sourceless=False):
                 # worth it.
                 pass
             assert sourceless in (
-                "pep3147_envonly", "simple", "pep3147_everything"), sourceless
+                "pep3147_envonly",
+                "simple",
+                "pep3147_everything",
+            ), sourceless
             make_sourceless(
                 os.path.join(path, "env.py"),
-                "pep3147" if "pep3147" in sourceless else "simple"
+                "pep3147" if "pep3147" in sourceless else "simple",
             )
 
     sc = script.ScriptDirectory.from_config(cfg)
@@ -53,40 +60,44 @@ def clear_staging_env():
 
 
 def script_file_fixture(txt):
-    dir_ = os.path.join(_get_staging_directory(), 'scripts')
+    dir_ = os.path.join(_get_staging_directory(), "scripts")
     path = os.path.join(dir_, "script.py.mako")
-    with open(path, 'w') as f:
+    with open(path, "w") as f:
         f.write(txt)
 
 
 def env_file_fixture(txt):
-    dir_ = os.path.join(_get_staging_directory(), 'scripts')
-    txt = """
+    dir_ = os.path.join(_get_staging_directory(), "scripts")
+    txt = (
+        """
 from alembic import context
 
 config = context.config
-""" + txt
+"""
+        + txt
+    )
 
     path = os.path.join(dir_, "env.py")
     pyc_path = util.pyc_file_from_path(path)
     if pyc_path:
         os.unlink(pyc_path)
 
-    with open(path, 'w') as f:
+    with open(path, "w") as f:
         f.write(txt)
 
 
 def _sqlite_file_db(tempname="foo.db"):
-    dir_ = os.path.join(_get_staging_directory(), 'scripts')
+    dir_ = os.path.join(_get_staging_directory(), "scripts")
     url = "sqlite:///%s/%s" % (dir_, tempname)
     return engines.testing_engine(url=url)
 
 
 def _sqlite_testing_config(sourceless=False):
-    dir_ = os.path.join(_get_staging_directory(), 'scripts')
+    dir_ = os.path.join(_get_staging_directory(), "scripts")
     url = "sqlite:///%s/foo.db" % dir_
 
-    return _write_config_file("""
+    return _write_config_file(
+        """
 [alembic]
 script_location = %s
 sqlalchemy.url = %s
@@ -115,14 +126,17 @@ keys = generic
 [formatter_generic]
 format = %%(levelname)-5.5s [%%(name)s] %%(message)s
 datefmt = %%H:%%M:%%S
-    """ % (dir_, url, "true" if sourceless else "false"))
+    """
+        % (dir_, url, "true" if sourceless else "false")
+    )
 
 
-def _multi_dir_testing_config(sourceless=False, extra_version_location=''):
-    dir_ = os.path.join(_get_staging_directory(), 'scripts')
+def _multi_dir_testing_config(sourceless=False, extra_version_location=""):
+    dir_ = os.path.join(_get_staging_directory(), "scripts")
     url = "sqlite:///%s/foo.db" % dir_
 
-    return _write_config_file("""
+    return _write_config_file(
+        """
 [alembic]
 script_location = %s
 sqlalchemy.url = %s
@@ -152,15 +166,22 @@ keys = generic
 [formatter_generic]
 format = %%(levelname)-5.5s [%%(name)s] %%(message)s
 datefmt = %%H:%%M:%%S
-    """ % (dir_, url, "true" if sourceless else "false",
-           extra_version_location))
+    """
+        % (
+            dir_,
+            url,
+            "true" if sourceless else "false",
+            extra_version_location,
+        )
+    )
 
 
 def _no_sql_testing_config(dialect="postgresql", directives=""):
     """use a postgresql url with no host so that
     connections guaranteed to fail"""
-    dir_ = os.path.join(_get_staging_directory(), 'scripts')
-    return _write_config_file("""
+    dir_ = os.path.join(_get_staging_directory(), "scripts")
+    return _write_config_file(
+        """
 [alembic]
 script_location = %s
 sqlalchemy.url = %s://
@@ -190,32 +211,36 @@ keys = generic
 format = %%(levelname)-5.5s [%%(name)s] %%(message)s
 datefmt = %%H:%%M:%%S
 
-""" % (dir_, dialect, directives))
+"""
+        % (dir_, dialect, directives)
+    )
 
 
 def _write_config_file(text):
     cfg = _testing_config()
-    with open(cfg.config_file_name, 'w') as f:
+    with open(cfg.config_file_name, "w") as f:
         f.write(text)
     return cfg
 
 
 def _testing_config():
     from alembic.config import Config
+
     if not os.access(_get_staging_directory(), os.F_OK):
         os.mkdir(_get_staging_directory())
-    return Config(os.path.join(_get_staging_directory(), 'test_alembic.ini'))
+    return Config(os.path.join(_get_staging_directory(), "test_alembic.ini"))
 
 
 def write_script(
-        scriptdir, rev_id, content, encoding='ascii', sourceless=False):
+    scriptdir, rev_id, content, encoding="ascii", sourceless=False
+):
     old = scriptdir.revision_map.get_revision(rev_id)
     path = old.path
 
     content = textwrap.dedent(content)
     if encoding:
         content = content.encode(encoding)
-    with open(path, 'wb') as fp:
+    with open(path, "wb") as fp:
         fp.write(content)
     pyc_path = util.pyc_file_from_path(path)
     if pyc_path:
@@ -223,20 +248,21 @@ def write_script(
     script = Script._from_path(scriptdir, path)
     old = scriptdir.revision_map.get_revision(script.revision)
     if old.down_revision != script.down_revision:
-        raise Exception("Can't change down_revision "
-                        "on a refresh operation.")
+        raise Exception(
+            "Can't change down_revision " "on a refresh operation."
+        )
     scriptdir.revision_map.add_revision(script, _replace=True)
 
     if sourceless:
         make_sourceless(
-            path,
-            "pep3147" if sourceless == "pep3147_everything" else "simple"
+            path, "pep3147" if sourceless == "pep3147_everything" else "simple"
         )
 
 
 def make_sourceless(path, style):
 
     import py_compile
+
     py_compile.compile(path)
 
     if style == "simple" and has_pep3147():
@@ -264,7 +290,10 @@ def three_rev_fixture(cfg):
 
     script = ScriptDirectory.from_config(cfg)
     script.generate_revision(a, "revision a", refresh=True)
-    write_script(script, a, """\
+    write_script(
+        script,
+        a,
+        """\
 "Rev A"
 revision = '%s'
 down_revision = None
@@ -279,10 +308,16 @@ def upgrade():
 def downgrade():
     op.execute("DROP STEP 1")
 
-""" % a)
+"""
+        % a,
+    )
 
     script.generate_revision(b, "revision b", refresh=True)
-    write_script(script, b, u("""# coding: utf-8
+    write_script(
+        script,
+        b,
+        u(
+            """# coding: utf-8
 "Rev B, méil, %3"
 revision = '{}'
 down_revision = '{}'
@@ -297,10 +332,16 @@ def upgrade():
 def downgrade():
     op.execute("DROP STEP 2")
 
-""").format(b, a), encoding="utf-8")
+"""
+        ).format(b, a),
+        encoding="utf-8",
+    )
 
     script.generate_revision(c, "revision c", refresh=True)
-    write_script(script, c, """\
+    write_script(
+        script,
+        c,
+        """\
 "Rev C"
 revision = '%s'
 down_revision = '%s'
@@ -315,7 +356,9 @@ def upgrade():
 def downgrade():
     op.execute("DROP STEP 3")
 
-""" % (c, b))
+"""
+        % (c, b),
+    )
     return a, b, c
 
 
@@ -328,8 +371,12 @@ def multi_heads_fixture(cfg, a, b, c):
 
     script = ScriptDirectory.from_config(cfg)
     script.generate_revision(
-        d, "revision d from b", head=b, splice=True, refresh=True)
-    write_script(script, d, """\
+        d, "revision d from b", head=b, splice=True, refresh=True
+    )
+    write_script(
+        script,
+        d,
+        """\
 "Rev D"
 revision = '%s'
 down_revision = '%s'
@@ -344,11 +391,17 @@ def upgrade():
 def downgrade():
     op.execute("DROP STEP 4")
 
-""" % (d, b))
+"""
+        % (d, b),
+    )
 
     script.generate_revision(
-        e, "revision e from d", head=d, splice=True, refresh=True)
-    write_script(script, e, """\
+        e, "revision e from d", head=d, splice=True, refresh=True
+    )
+    write_script(
+        script,
+        e,
+        """\
 "Rev E"
 revision = '%s'
 down_revision = '%s'
@@ -363,11 +416,17 @@ def upgrade():
 def downgrade():
     op.execute("DROP STEP 5")
 
-""" % (e, d))
+"""
+        % (e, d),
+    )
 
     script.generate_revision(
-        f, "revision f from b", head=b, splice=True, refresh=True)
-    write_script(script, f, """\
+        f, "revision f from b", head=b, splice=True, refresh=True
+    )
+    write_script(
+        script,
+        f,
+        """\
 "Rev F"
 revision = '%s'
 down_revision = '%s'
@@ -382,7 +441,9 @@ def upgrade():
 def downgrade():
     op.execute("DROP STEP 6")
 
-""" % (f, b))
+"""
+        % (f, b),
+    )
 
     return d, e, f
 
@@ -390,18 +451,16 @@ def downgrade():
 def _multidb_testing_config(engines):
     """alembic.ini fixture to work exactly with the 'multidb' template"""
 
-    dir_ = os.path.join(_get_staging_directory(), 'scripts')
+    dir_ = os.path.join(_get_staging_directory(), "scripts")
 
-    databases = ", ".join(
-        engines.keys()
-    )
+    databases = ", ".join(engines.keys())
     engines = "\n\n".join(
-        "[%s]\n"
-        "sqlalchemy.url = %s" % (key, value.url)
+        "[%s]\n" "sqlalchemy.url = %s" % (key, value.url)
         for key, value in engines.items()
     )
 
-    return _write_config_file("""
+    return _write_config_file(
+        """
 [alembic]
 script_location = %s
 sourceless = false
@@ -432,5 +491,6 @@ keys = generic
 [formatter_generic]
 format = %%(levelname)-5.5s [%%(name)s] %%(message)s
 datefmt = %%H:%%M:%%S
-    """ % (dir_, databases, engines)
+    """
+        % (dir_, databases, engines)
     )
